
! The code was developed at the Fritz Haber Institute, and
! the intellectual properties and copyright of this file
! are with the Max Planck Society. When you use it, please
! cite R. Gomez-Abal, X. Li, C. Ambrosch-Draxl, M. Scheffler,
! Extended linear tetrahedron method for the calculation of q-dependent
! dynamical response functions, to be published in Comp. Phys. Commun. (2010)

!BOP
! 
! !ROUTINE: kgenq
!
! !INTERFACE:
      subroutine kgenq(rbas,ndiv,nshift,divsh,qv,nkpt,&
     &                klist,idiv,wk,ntet,tetk,linkt,vtet)
!
! !DESCRIPTION:
!
! This subroutine generates the set of irreducible k-points for tetrahedron
!integration of operators depending on a vector q. The k-point mesh is
!identical to that generated by kgen. But the tetrahedra can not be reduced
!by symmetry (except for q=0)
!
! The list of k-points are given in
! integer internal coordinates of the reciprocal lattice. The symmetry
! operations must also be given to the subroutine in internal coordinates.
! If the calling program is using cartesian coordinates (as is the case in
! the WIEN2k package, when \verb"ortho" = .\verb"true".) the subroutine
! \verb"sym2int" (also contained in this library) has to be called before
!to transform the symmetry operations to internal coordinates and the
!subroutine \verb"cartezian" has to be called afterwards to transform the
!k-points coordinates back to cartesian ones.
!
! Since the dimension of most of the output parameters are determined
! within the subroutine, these dummy array arguments are allocatable, requiring an
! explicit interface to be written in the calling program.
!
     
! !USES:

      use kgen_internals     

      implicit none

! !INPUT PARAMETERS:

      real(8), dimension(3,3), intent(in) :: rbas  ! Basis vectors of the
!                                                     Bravais lattice
      
      integer(4), dimension(3), intent(in) :: ndiv ! Number of divisions
!                                                    of the BZ in each 
!                                                    direction

      integer(4), dimension(3), intent(in) :: nshift ! Shift vector for 
!                                                      the k-points mesh
      integer(4),intent(in) :: divsh

      integer(4), dimension(3), intent(in) :: qv  ! Shift vector for the
!                                                    k-points mesh
      integer(4), intent(in) :: nkpt        ! Number of irreducible
!                                             k-points

! !OUTPUT PARAMETERS:


      integer(4), intent(out) :: klist(3,nkpt) ! The list of 
!                                                           irreducible 
!                                                           k-points

      integer(4), intent(out) :: idiv       ! Minimum common divisor or
!                                             the integer sublattice 
!                                             coordinates of the
!                                             irreducible k-points

      integer(4), intent(out) :: wk(nkpt)   ! The weight of 
!                                                       each k-point
      
      integer(4), intent(out) :: ntet       ! Number of tetrahedra

      integer(4), intent(out) :: tetk(4,nkpt*6)  ! id. number
!                                                          of the 
!                                                          k-points 
!                                                          corresponding 
!                                                          to the nodes 
!                                                          of each 
!                                                          tetrahedron.

      integer(4), intent(out) :: linkt(nkpt*6)   ! index of the 
!                                                          tetrahedron 
!                                                          linked to the
!                                                          one in the
!                                                          index by the 
!                                                          vector q

      real(8),    intent(out) :: vtet      ! volume of each tetrahedron
      


! !LOCAL VARIABLES:

      integer(4) :: i,j                     ! Just some counters
      
      integer(4), dimension(3) :: onek       ! Temporary storage of one
!                                             k-point coordinates
 
      integer(4), allocatable  :: kpt(:,:) ! Coordinates of the k-points
      integer(4), dimension(3) :: q         ! Vector q for which the set
!                                             of(k,k') points are 
!                                             calculated
 
      

!EOP
!
!BOC
      div=ndiv
      shift=nshift
!
!     Set the total number of k-points
!
!      nkpt=ndiv(1)*ndiv(2)*ndiv(3)
!
!     Ensure that q is inside the BZ
! 
      allocate(kpt(3,nkpt))
      do i=1,3
        q(i)=mod(qv(i)+(1-isign(1,qv(i)))/2*ndiv(i),ndiv(i))
      enddo

!      call reduk(nsymt,wt)

!
!     Allocate arrays depending on nirkp
!      

      wk(1:nkpt)=1
!
!     Calculate the sublattice coordinates of the irreducible k-points
!      
      do i=1,nkpt
        call coorskp(i,onek)
        do j=1,3
          kpt(j,i)=onek(j)
        enddo
      enddo
!
!     Transform the irreducible k-points to internal coordinates
!     
      print *,'nkpt=',nkpt
      call intern(nkpt,kpt,ndiv,nshift,divsh, klist,idiv)
           
      print *,'nkpt=',nkpt

     
      call gbass(rbas,gbas)
!
!     Generate the tetrahedra
!       
      call tgenq(q,ntet,tetk,linkt)
      vtet=vt
      deallocate(kpt)

      print *,'nkpt=',nkpt
      return

      end subroutine kgenq
!EOC
